/**
 * FLL - Level 3
 *
 * Project: FSS Write
 * API Version: 0.7
 * Licenses: lgpl-2.1-or-later
 *
 * Provides the common data structures.
 *
 * This is auto-included and should not need to be explicitly included.
 */
#ifndef _fss_write_common_type_h
#define _fss_write_common_type_h

#ifdef __cplusplus
extern "C" {
#endif

/**
 * The FSS write callbacks.
 *
 * process_content: Process a single Content.
 * process_help:    Process help (generally printing help).
 * process_normal:  Process normally (data from parameters and files).
 * process_object:  Process a single Object.
 * process_pipe:    Process using the data from input pipe.
 * process_set:     Process a set of Object and one or more Content.
 */
#ifndef _di_fss_write_callback_t_
  typedef struct {
    void (*process_content)(void * const main, const bool last);
    void (*process_help)(void * const main);
    void (*process_normal)(void * const main);
    void (*process_object)(void * const main);
    void (*process_pipe)(void * const main);
    void (*process_set)(void * const main);
  } fss_write_callback_t;

  #define fss_write_callback_t_initialize \
    { \
      0, \
      0, \
      0, \
      0, \
      0, \
      0, \
    }
#endif // _di_fss_write_callback_t_

/**
 * The fss write main program settings.
 *
 * This is passed to the program-specific main entry point to designate program settings.
 * These program settings are often processed from the program arguments (often called the command line arguments).
 *
 * For all function pointers on this structure, the main variable must be of type fss_write_main_t.
 *
 * flag: Flags passed to the main function.
 *
 * status_signal: A status used eclusively by the threaded signal handler.
 * state:         The state data used when processing the FSS data.
 *
 * range: A range used in conjunction with some buffer during processing.
 *
 * quote:    This holds the quote used during processing.
 * standard: A human-friendly string describing the standard in use, such as "FSS-0000 (Basic)".
 *
 * escaped: A buffer used for escaping strings during processing.
 * block:   A buffer used to storing one or more blocks while processing a file line by line.
 * buffer:  A buffer used during processing the file.
 * prepend: A string to prepend to each multi-line Content.
 *
 * ignoress:  An array of range sets passed as values to the "--ignore" parameter or via the input pipe.
 * objects:   An array of objects passed as values to the "--object" parameter or via the input pipe.
 * contentss: An array of content sets passed as values to the "--content" parameter or via the input pipe.
 *
 * ignores:  A pointer to a specific Ignores used during processing.
 * object:   A pointer to a specific Object used during processing.
 * content:  A pointer to a specific Content used during processing.
 * contents: A pointer to a specific set of Content used during processing.
 */
#ifndef _di_fss_write_setting_t_
  typedef struct {
    uint32_t flag;

    f_status_t status_signal;
    f_state_t state;

    f_range_t range;

    f_string_static_t quote;
    f_string_static_t standard;

    f_string_dynamic_t block;
    f_string_dynamic_t buffer;
    f_string_dynamic_t prepend;

    f_rangess_t ignoress;
    f_string_dynamics_t objects;
    f_string_dynamicss_t contentss;

    f_ranges_t *ignores;
    f_string_static_t *object;
    f_string_static_t *content;
    f_string_statics_t *contents;
  } fss_write_setting_t;

  #define fss_write_setting_t_initialize \
    { \
      fss_write_main_flag_none_d, \
      F_okay, \
      macro_f_state_t_initialize_1(fss_write_allocation_large_d, fss_write_allocation_small_d, F_okay, 0, 0, &fll_program_standard_signal_handle, 0, 0, 0, 0), \
      f_range_t_initialize, \
      f_string_static_t_initialize, \
      f_string_static_t_initialize, \
      f_string_dynamic_t_initialize, \
      f_string_dynamic_t_initialize, \
      f_string_dynamic_t_initialize, \
      f_rangess_t_initialize, \
      f_string_dynamics_t_initialize, \
      f_string_dynamicss_t_initialize, \
      0, \
      0, \
      0, \
      0, \
    }
#endif // _di_fss_write_setting_t_

/**
 * The main program data as a single structure.
 *
 * callback: The callback data.
 * program:  The main program data.
 * setting:  The settings data.
 */
#ifndef _di_fss_write_main_t_
  typedef struct {
    fss_write_callback_t callback;
    fll_program_data_t program;
    fss_write_setting_t setting;
  } fss_write_main_t;

  #define fss_write_main_t_initialize \
    { \
      fss_write_callback_t_initialize, \
      fll_program_data_t_initialize, \
      fss_write_setting_t_initialize, \
    }
#endif // _di_fss_write_main_t_

/**
 * De-allocate main program data.
 *
 * @param main
 *   The main program data.
 *
 *   Must not be NULL.
 *
 *   This does not alter main.setting.state.status.
 *
 * @see fll_program_data_delete()
 * @see fss_write_setting_delete()
 */
#ifndef _di_fss_write_main_delete_
  extern void fss_write_main_delete(fss_write_main_t * const main);
#endif // _di_fss_write_main_delete_

/**
 * Delete the program main setting data.
 *
 * @param setting
 *   The program main setting data.
 *
 *   Must not be NULL.
 *
 *   This does not alter setting.state.status.
 *
 * @see f_memory_array_resize()
 * @see f_memory_arrays_resize()
 */
#ifndef _di_fss_write_setting_delete_
  extern void fss_write_setting_delete(fss_write_setting_t * const setting);
#endif // _di_fss_write_setting_delete_

#ifdef __cplusplus
} // extern "C"
#endif

#endif // _fss_write_common_type_h
